<?php
/*
 * Plugin Name: WooCommerce Pagadito Payment Gateway
 * Plugin URI: https://wordpress.org/plugins/woo-pagadito-payment-gateway/
 * Description: Pagadito le permite Pagar en línea de una manera segura, fácil y confiable.
 * Version: 1.3.1
 * Author: Pagadito Developers
 * Author URI: https://dev.pagadito.com/
 * License: LGPL3
 *
 * Text Domain: woocommerce-gateway-pagadito
 * Domain Path: /languages
*/

/*  Copyright (c) 2019 PAGADITO, S. DE R.L. (developers@pagadito.com)

    Este programa es Software Libre: Usted puede redistribuirlo y/o modificarlo
    bajo los términos de la Licencia Pública General Reducida de GNU (GNU Lesser
    General Public Licence), versión 3 ó superior, tal como se encuentra publicada
    por la Free Software Foundation.

    Este programa se distribuye con la esperanza de que sea útil, pero SIN NINGUNA
    GARANTIA; sin siquiera la garantía implícita de MERCANTIBILIDAD o ADAPTABILIDAD
    PARA UN PROPÓSITO PARTICULAR. Vea la Licencia Pública General Reducida de GNU
    para más detalles.

    Debería haber recibido una copia de la Licencia Pública General Reducida de GNU
    junto con este programa; sino escriba a la Free Software Foundation, Inc.,
    51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

require_once 'includes/pagadito-1.5.1.php';

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

/**
 * Notificación donde muestra que debe estar instalado y activo WooCommerce
 * @return string
 */
function woocommerce_pagadito_missing_wc_notice() {
    echo '<div class="error"><p><strong>' . sprintf( esc_html__( 'Pagadito requiere que WooCommerce este instalado y activo. Descarga %s aqui.', 'woocommerce-gateway-pagadito' ), '<a href="https://woocommerce.com/" target="_blank">WooCommerce</a>' ) . '</strong></p></div>';
}

add_action( 'plugins_loaded', 'woocommerce_pagadito_init' );

function woocommerce_pagadito_init() {
    load_plugin_textdomain( 'woocommerce-gateway-pagadito', false, plugin_basename( dirname( __FILE__ ) ) . '/languages' );

    if ( ! class_exists( 'WooCommerce' ) ) {
        add_action( 'admin_notices', 'woocommerce_pagadito_missing_wc_notice' );
        return;
    }

    if ( class_exists( 'Pagadito' ) ) :
        class WC_Gateway_Pagadito extends WC_Payment_Gateway {
            public function __construct(){
                global $woocommerce;
                
                $this->id                 = 'pagadito';
                $this->has_fields         = false;
                $this->method_title       = 'Pagadito';
                $this->method_description = __( 'Pagadito redirige a los clientes al sitio web de Pagadito, donde deben introducir su informaci&oacute;n de pago, posteriormente el cliente es devuelto a su sitio web para completar la orden.', 'woocommerce-gateway-pagadito' );
                
                $this->init_form_fields();
                $this->init_settings();
                
                /* Cargando configuraciones */
                $this->title              = $this->get_option( 'title' );
                $this->icon_type          = $this->get_option( 'icon_type' );
                $this->description        = $this->get_option( 'description' );
                $this->uid                = $this->get_option( 'uid' );
                $this->wsk                = $this->get_option( 'wsk' );
                $this->sandbox_mode       = $this->get_option( 'sandbox_mode' );
                $this->debug              = $this->get_option( 'debug' );
                $this->icon               = apply_filters( 'woocommerce_pagadito_icon', $woocommerce->plugin_url() . '/../woo-pagadito-payment-gateway/assets/images/' . $this->icon_type . '.png' );
                
                // hook para opciones administrativas
                add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( &$this, 'process_admin_options' ) );
                // hook para respuesta de pagadito
                add_action( 'woocommerce_api_wc_gateway_pagadito', array( $this, 'check_pagadito_response' ) );
                // web hook para respuesta de pagadito
                add_action( 'woocommerce_api_wc_webhook_pagadito', array( $this, 'pagadito_webhook' ) );
                
                // Desactivar si no esta disponible alguna de las monedas oficiales de pagadito
                if ( ! $this->is_valid_for_use() ) $this->enabled = false;
            }

            /**
             * Registrar en Log
             * Verifica antes si debug esta activo para permitir guardar en en archivo
             * @param string $message Información a registrar en log
             */
            public function save_to_log( $order, $message ) {
                if ( $this->debug ) {
                    if ( empty( $this->log ) ) {
                        $this->log = new WC_Logger();
                    }
                    $this->log->add( 'pagadito', 'Orden: ' . $order . ' - ' . $message );
                }
            }

            public function init_form_fields(){
                $this->form_fields = array(
                    'enabled' => array(
                        'title'       => __( 'Activar/Desactivar', 'woocommerce-gateway-pagadito' ),
                        'type'        => 'checkbox',
                        'label'       => __( 'Activar pagos con Pagadito', 'woocommerce-gateway-pagadito' ),
                        'default'     => 'yes'
                    ),
                    'title' => array(
                        'title'       => __( 'T&iacute;tulo', 'woocommerce-gateway-pagadito' ),
                        'type'        => 'text',
                        'description' => __( 'Controla el t&iacute;tulo que el usuario v&eacute; en el proceso de pago, si se encuentra vac&iacute;o s&oacute;lo aparecer&aacute; el logo de Pagadito.', 'woocommerce-gateway-pagadito' ),
                        'default'     => 'Pagadito',
                        'desc_tip'    => true,
                    ),
                    'description' => array(
                        'title'       => __( 'Descripci&oacute;n', 'woocommerce-gateway-pagadito' ),
                        'type'        => 'text',
                        'description' => __( 'Controla la descripci&oacute;n que el usuario v&eacute; en el proceso de pago.', 'woocommerce-gateway-pagadito' ),
                        'default'     => __( 'Pagadito le permite Pagar en l&iacute;nea de una manera segura, f&aacute;cil y confiable.', 'woocommerce-gateway-pagadito' ),
                        'desc_tip'    => true,
                    ),
                    'icon_type' => array(
                        'title'       => __( 'Logo Pagadito', 'woocommerce-gateway-pagadito' ),
                        'type'        => 'select',
                        'options'     => array(
                            'color'   => __( 'A colores', 'woocommerce-gateway-pagadito' ),
                            'white'   => __( 'Blanco', 'woocommerce-gateway-pagadito' ),
                            'black'   => __( 'Negro', 'woocommerce-gateway-pagadito' ),
                        ),
                        'description' => __( 'Controla el tema del logo de Pagadito que aparecer&aacute; en la pantalla de pago.', 'woocommerce-gateway-pagadito' ),
                        'default'     => 'color',
                    ),
                    'options_details' => array(
                        'title'       => __( 'Opciones avanzadas', 'woocommerce-gateway-pagadito' ),
                        'type'        => 'title',
                    ),
                    'sandbox_mode' => array(
                        'title'       => 'SandBox Pagadito',
                        'type'        => 'checkbox',
                        'label'       => __( 'Activar el entorno de pruebas de Pagadito', 'woocommerce-gateway-pagadito' ),
                        'description' => sprintf( __( 'Sandbox Pagadito es una herramienta que permite realizar pruebas de Pagadito para probar pagos. Crea una %sCuenta Pagadito Comercios en SandBox%s. Una vez finalizadas las pruebas aseg&uacute;rese de deshabilitar esta opci&oacute;n, para procesar pagos en entorno de Producci&oacute;n.', 'woocommerce-gateway-pagadito' ), '<a href="https://sandbox.pagadito.com/index.php?mod=user&hac=vregfC" target="_blank">', '</a>' ),
                        'default'     => 'yes', 
                    ),
                    'debug' => array(
                        'title'       => __( 'Registro de depuraci&oacute;n', 'woocommerce-gateway-pagadito' ),
                        'type'        => 'checkbox',
                        'label'       => __( 'Permitir registro de depuraci&oacute;n', 'woocommerce-gateway-pagadito' ),
                        'description' => sprintf( __( 'Eventos de registro del API, puede encontrarlos en el archivo <code>%s</code>', 'woocommerce-gateway-pagadito' ), wc_get_log_file_path( 'pagadito' ) ),
                        'default'     => 'yes',
                    ),
                    'api_details' => array(
                        'title'       => __( 'Par&aacute;metros de integraci&oacute;n', 'woocommerce-gateway-pagadito' ),
                        'type'        => 'title',
                        'description' => sprintf( __( 'Aprende como acceder a tus %sCredenciales de conexi&oacute;n%s.', 'woocommerce-gateway-pagadito' ), '<a href="https://dev.pagadito.com/index.php?mod=docs&hac=conf" target="_blank">', '</a>' ),
                    ),
                    'uid' => array(
                        'title'       => 'UID',
                        'type'        => 'text',
                        'description' => __( 'Representa el identificador de su cuenta Pagadito Comercio.', 'woocommerce-gateway-pagadito' ),
                        'default'     => '',
                        'desc_tip'    => true,
                    ),
                    'wsk' => array(
                        'title'       => 'WSK',
                        'type'        => 'text',
                        'description' => __( 'Representa la clave de conexi&oacute;n para conectarse a Pagadito.', 'woocommerce-gateway-pagadito' ),
                        'default'     => '',
                        'desc_tip'    => true,
                    ),'url_return' => array(
                        'title'       => __( 'URL de Retorno', 'woocommerce-gateway-pagadito' ),
                        'type'        => 'title',
                        'description' => sprintf( __( 'Ingrese esta direcci&oacute;n <code>%s</code> en el campo URL de Retorno en su cuenta de Pagadito -> Configuraci&oacute;n t&eacute;cnica -> Par&aacute;metros de integraci&oacute;n.', 'woocommerce-gateway-pagadito' ), home_url() . '/?wc-api=WC_Gateway_Pagadito&token={value}&order_id={ern_value}' ),
                    ),
                    'url_webhook' => array(
                        'title'       => __( 'Webhooks', 'woocommerce-gateway-pagadito' ),
                        'type'       => 'title',
                        'description' => sprintf( __( 'Para una integraci&oacute;n completa con Pagadito, aseg&uacute;rese de habilitar el env&iacute;o de notificaciones v&iacute;a Webhooks e ingresar esta direcci&oacute;n <code>%s</code> en el campo de Webhook URL en su cuenta de Pagadito -> Configuraci&oacute;n t&eacute;cnica -> Webhooks.', 'woocommerce-gateway-pagadito' ), home_url(). '/?wc-api=WC_Webhook_Pagadito' ),
                    ),
                );
            }
            
            private function is_valid_for_use() {
                if ( ! in_array( get_woocommerce_currency(), apply_filters( 'woocommerce_pagadito_supported_currencies', array( 'USD', 'GTQ', 'HNL', 'NIO', 'CRC', 'PAB', 'DOP' ) ) ) ) return false;

                return true;
            }
            /**
             * Procesar Pago
             * @param integer $order_id La orden que será procesada por Pagadito
             * @return array Devuelve la url a la que debe direccionar, en caso de error lo muestra en pantalla
             */
            public function process_payment( $order_id ){
                global $woocommerce;
                $order = new WC_Order( $order_id );
                
                $tax_included = get_option( 'woocommerce_prices_include_tax' == 'yes' ) ? true: false;
                
                try{
                    $this->save_to_log( $order->get_order_number(), '' );
                    $this->save_to_log( $order->get_order_number(), 'Procesando Pago' );

                    //Inicializando api de pagadito
                    $pg = new Pagadito( $this->uid, $this->wsk );

                    if ( $this->sandbox_mode == 'yes' ) {
                        $pg->mode_sandbox_on();
                        $this->save_to_log( $order->get_order_number(), 'Transaccion en modo SandBox activado' );
                    }

                    if ( ! $pg->connect() )
                        throw new Exception( 'Connect (' . $pg->get_rs_code() . '): ' . $pg->get_rs_message() );

                    $pg->enable_pending_payments();
                    $token_pagadito = $pg->get_rs_value();
                    $this->save_to_log( $order->get_order_number(), 'Token de conexion = ' . $pg->get_rs_value() );
                    
                    $moneda = strtoupper( get_woocommerce_currency() );
                    $this->save_to_log( $order->get_order_number(), 'La orden se procesa en moneda = ' . $moneda );

                    //Cambiando tipo de moneda
                    switch ( $moneda ) {
                        case 'USD':
                            $pg->change_currency_usd();
                            break;
                        case 'GTQ':
                            $pg->change_currency_gtq();
                            break;
                        case 'HNL':
                            $pg->change_currency_hnl();
                            break;
                        case 'NIO':
                            $pg->change_currency_nio();
                            break;
                        case 'CRC':
                            $pg->change_currency_crc();
                            break;
                        case 'PAB':
                            $pg->change_currency_pab();
                            break;
                        case 'DOP':
                            $pg->change_currency_dop();
                            break;
                    }
                    
                    if ( $order->get_item_count() <= 0 )
                        throw new Exception( __( 'No hay items en la orden', 'woocommerce-gateway-pagadito' ) );
                    
                    // Agregando items
                    foreach ( $order->get_items() as $item ) {
                        if ( $tax_included )
                            $pg->add_detail( $item[ 'qty' ], $item[ 'name' ], $order->get_item_subtotal( $item, true, false ) );
                        else
                            $pg->add_detail( $item[ 'qty' ], $item[ 'name' ], $order->get_item_subtotal( $item, false, true ) );
                    }
                    
                    // Aplicando fees
                    $fees = $order->get_fees();
                    if ( is_array( $fees ) && count( $fees ) > 0 ) {
                        foreach ( $fees as $fee ) {
                            $pg->add_detail( 1, $fee[ 'name' ], $fee[ 'line_total' ] );
                        }
                    }
                                      
                    // Aplicando costos de envio (si aplica)
                    $shipping = $order->get_total_shipping();
                    $shipping_tax = $order->get_shipping_tax();

                    if ( ( abs( $shipping ) - 0) >= 0.01 ) {
                        $pg->add_detail( 1, __( 'Env&iacute;o v&iacute;a: ', 'woocommerce-gateway-pagadito' ) . ucwords( $order->get_shipping_method() ), $shipping );
                    }
                    
                    // Aplicando impuestos (si los hay)
                    $impuestos = $order->get_total_tax();
                    if ( ( abs( $impuestos ) - 0 ) >= 0.01 ) {
                        $pg->add_detail( 1, __( 'Total de Impuestos: ', 'woocommerce-gateway-pagadito' ), $impuestos );
                    }

                    // Aplicando descuento
                    if ( ! empty( $woocommerce->cart->applied_coupons ) ) {
                        $discount = $order->get_discount_total();

                        if ( ( abs( $discount ) - 0 ) >= 0.01 ) {
                            $discount = abs( $discount ) * ( -1 );
                            $pg->add_detail( 1, __( 'Cup&oacute;n descuento (', 'woocommerce-gateway-pagadito' ) . implode( '+', $order->get_used_coupons() ) . ')', $discount );
                        }
                    }

                    // Validando que la transaccion sea mayor a un dolar incluyendo sus diferentes conversiones
                    switch( $moneda ){
                        case "USD":
                            if ( $order->get_total() >= 1.00 ) {

                                //Continuar con Pagadito
                                $this->save_to_log( $order->get_order_number(), 'Continuando con Pagadito...' );
                                $redirect_url = $pg->exec_trans( $order->get_id() );
                                
                                if ( $redirect_url === false ) {
                                    throw new Exception( 'Exec trans (' . $pg->get_rs_code() . '): ' . $pg->get_rs_message() );
                                }

                                // Proceso realizado con exito
                                $result = array(
                                    'result' => 'success',
                                    'redirect' => $redirect_url
                                );
                                
                                // Asociando transaccion pagadito con la orden
                                add_post_meta( $order->get_id(), '_token_pagadito', $token_pagadito );

                                return $result;
                            }
                            else {
                                $this->save_to_log( $order->get_order_number(), 'La transaccion es menor a 1 dolar = ' . $order->get_total() . $moneda );
                                wc_add_notice( __( '<strong>Nota:</strong> No se pueden procesar las transacciones menores a 1 d&oacute;lar', 'woocommerce-gateway-pagadito' ), 'notice' );
                            }
                            break;
                        case "GTQ":
                            if ( $order->get_total() >= $pg->get_exchange_rate_gtq() ) {

                                //Continuar con Pagadito
                                $this->save_to_log( $order->get_order_number(), 'Continuando con Pagadito...' );
                                $redirect_url = $pg->exec_trans( $order->get_id() );
                                
                                if ( $redirect_url === false ) {
                                    throw new Exception( 'Exec trans (' . $pg->get_rs_code() . '): ' . $pg->get_rs_message() );
                                }

                                // Proceso realizado con exito
                                $result = array(
                                    'result' => 'success',
                                    'redirect' => $redirect_url
                                );
                                
                                // Asociando transaccion pagadito con la orden
                                add_post_meta( $order->get_id(), '_token_pagadito', $token_pagadito );

                                return $result;
                            }
                            else {
                                $this->save_to_log( $order->get_order_number(), 'La transaccion es menor a 1 dolar = ' . $order->get_total() . $moneda );
                                wc_add_notice( __( '<strong>Nota:</strong> No se pueden procesar las transacciones menores al equivalente a 1 d&oacute;lar', 'woocommerce-gateway-pagadito' ), 'notice' );
                            }
                            break;
                        case "HNL":
                            if ( $order->get_total() >= $pg->get_exchange_rate_hnl() ) {

                                //Continuar con Pagadito
                                $this->save_to_log( $order->get_order_number(), 'Continuando con Pagadito...' );
                                $redirect_url = $pg->exec_trans( $order->get_id() );
                                
                                if ( $redirect_url === false ) {
                                    throw new Exception( 'Exec trans (' . $pg->get_rs_code() . '): ' . $pg->get_rs_message() );
                                }

                                // Proceso realizado con exito
                                $result = array(
                                    'result' => 'success',
                                    'redirect' => $redirect_url
                                );
                                
                                // Asociando transaccion pagadito con la orden
                                add_post_meta( $order->get_id(), '_token_pagadito', $token_pagadito );

                                return $result;
                            }
                            else {
                                $this->save_to_log( $order->get_order_number(), 'La transaccion es menor a 1 dolar = ' . $order->get_total() . $moneda );
                                wc_add_notice( __( '<strong>Nota:</strong> No se pueden procesar las transacciones menores al equivalente a 1 d&oacute;lar', 'woocommerce-gateway-pagadito' ), 'notice' );
                            }
                            break;
                        case "NIO":
                            if ( $order->get_total() >= $pg->get_exchange_rate_nio() ) {

                                //Continuar con Pagadito
                                $this->save_to_log( $order->get_order_number(), 'Continuando con Pagadito...' );
                                $redirect_url = $pg->exec_trans( $order->get_id() );
                                
                                if ( $redirect_url === false ) {
                                    throw new Exception( 'Exec trans (' . $pg->get_rs_code() . '): ' . $pg->get_rs_message() );
                                }

                                // Proceso realizado con exito
                                $result = array(
                                    'result' => 'success',
                                    'redirect' => $redirect_url
                                );
                                
                                // Asociando transaccion pagadito con la orden
                                add_post_meta( $order->get_id(), '_token_pagadito', $token_pagadito );

                                return $result;
                            }
                            else {
                                $this->save_to_log( $order->get_order_number(), 'La transaccion es menor a 1 dolar = ' . $order->get_total() . $moneda );
                                wc_add_notice( __( '<strong>Nota:</strong> No se pueden procesar las transacciones menores al equivalente a 1 d&oacute;lar', 'woocommerce-gateway-pagadito' ), 'notice' );
                            }
                            break;
                        case "CRC":
                            if ( $order->get_total() >= $pg->get_exchange_rate_crc() ) {

                                //Continuar con Pagadito
                                $this->save_to_log( $order->get_order_number(), 'Continuando con Pagadito...' );
                                $redirect_url = $pg->exec_trans( $order->get_id() );
                                
                                if ( $redirect_url === false ) {
                                    throw new Exception( 'Exec trans (' . $pg->get_rs_code() . '): ' . $pg->get_rs_message() );
                                }

                                // Proceso realizado con exito
                                $result = array(
                                    'result' => 'success',
                                    'redirect' => $redirect_url
                                );
                                
                                // Asociando transaccion pagadito con la orden
                                add_post_meta( $order->get_id(), '_token_pagadito', $token_pagadito );

                                return $result;
                            }
                            else {
                                $this->save_to_log( $order->get_order_number(), 'La transaccion es menor a 1 dolar = ' . $order->get_total() . $moneda );
                                wc_add_notice( __( '<strong>Nota:</strong> No se pueden procesar las transacciones menores al equivalente a 1 d&oacute;lar', 'woocommerce-gateway-pagadito' ), 'notice' );
                            }
                            break;
                        case "PAB":
                            if ( $order->get_total() >= $pg->get_exchange_rate_pab() ) {

                                //Continuar con Pagadito
                                $this->save_to_log( $order->get_order_number(), 'Continuando con Pagadito...' );
                                $redirect_url = $pg->exec_trans( $order->get_id() );
                                
                                if ( $redirect_url === false ) {
                                    throw new Exception( 'Exec trans (' . $pg->get_rs_code() . '): ' . $pg->get_rs_message() );
                                }

                                // Proceso realizado con exito
                                $result = array(
                                    'result' => 'success',
                                    'redirect' => $redirect_url
                                );
                                
                                // Asociando transaccion pagadito con la orden
                                add_post_meta( $order->get_id(), '_token_pagadito', $token_pagadito );

                                return $result;
                            }
                            else {
                                $this->save_to_log( $order->get_order_number(), 'La transaccion es menor a 1 dolar = ' . $order->get_total() . $moneda );
                                wc_add_notice( __( '<strong>Nota:</strong> No se pueden procesar las transacciones menores al equivalente a 1 d&oacute;lar', 'woocommerce-gateway-pagadito' ), 'notice' );
                            }
                            break;
                        case "DOP":
                            if ( $order->get_total() >= $pg->get_exchange_rate_dop() ) {

                                //Continuar con Pagadito
                                $this->save_to_log( $order->get_order_number(), 'Continuando con Pagadito...' );
                                $redirect_url = $pg->exec_trans( $order->get_id() );
                                
                                if ( $redirect_url === false ) {
                                    throw new Exception( 'Exec trans (' . $pg->get_rs_code() . '): ' . $pg->get_rs_message() );
                                }

                                // Proceso realizado con exito
                                $result = array(
                                    'result' => 'success',
                                    'redirect' => $redirect_url
                                );
                                
                                // Asociando transaccion pagadito con la orden
                                add_post_meta( $order->get_id(), '_token_pagadito', $token_pagadito );

                                return $result;
                            }
                            else {
                                $this->save_to_log( $order->get_order_number(), 'La transaccion es menor a 1 dolar = ' . $order->get_total() . $moneda );
                                wc_add_notice( __( '<strong>Nota:</strong> No se pueden procesar las transacciones menores al equivalente a 1 d&oacute;lar', 'woocommerce-gateway-pagadito' ), 'notice' );
                            }
                            break;
                    }
                } catch( Exception $e ){
                    $this->save_to_log( $order->get_order_number(), 'Exception = ' . $e->getMessage() . ' | CURL error = ' . $pg->curl_errorno );
                    wc_add_notice( __( '<strong>Ha ocurrido un problema:</strong> ', 'woocommerce-gateway-pagadito' ) . $e->getMessage(), 'error' );
                }
            }
            
            public function pagadito_webhook(){
                // obtener headers
                $headers = getallheaders();
                $successful = false;
                
                if ( ! isset(
                    $headers[ 'PAGADITO-NOTIFICATION-ID' ],
                    $headers[ 'PAGADITO-NOTIFICATION-TIMESTAMP' ],
                    $headers[ 'PAGADITO-AUTH-ALGO' ],
                    $headers[ 'PAGADITO-CERT-URL' ],
                    $headers[ 'PAGADITO-SIGNATURE'] )
                ) {
                    header( 'HTTP/1.1 428 Nothing to see here' );
                    exit;
                }
                
                $notification_id = $headers[ 'PAGADITO-NOTIFICATION-ID' ];
                $notification_timestamp = $headers[ 'PAGADITO-NOTIFICATION-TIMESTAMP' ];
                $auth_algo = $headers[ 'PAGADITO-AUTH-ALGO' ];
                $cert_url = $headers[ 'PAGADITO-CERT-URL' ];
                $notification_signature = base64_decode( $headers[ 'PAGADITO-SIGNATURE' ] );
                
                // verificando que la url del certificado esta bajo pagadito
                $cent_url_parsed = parse_url( $cert_url );
                $cert_url_exploded = explode( '.', $cent_url_parsed[ 'host' ] );
                if ( $cert_url_exploded[ count( $cert_url_exploded ) - 2 ] != 'pagadito' ){
                    header( 'HTTP/1.1 429 Naughty boy' );
                    exit();
                }

                // obtener data
                $data = @file_get_contents( 'php://input' );

                // obtener id evento
                $obj_data = @json_decode( $data );
                $event_id = $obj_data->id;

                // generar cadena para confirmar firma
                $data_signed = $notification_id . '|' . $notification_timestamp . '|' . $event_id . '|' . crc32($data) . '|' . $this->wsk;

                // obtener contenido del certificado
                // opciones de peticiones http para generar el stream context para obtener el certificado
                $http_options = array(
                    'http' => array(
                        'protocol_version' => '1.1',
                        'method' => 'GET',
                        'header' => array(
                            'Connection: close'
                        ),
                    )
                );
                $cert_stream_context = stream_context_create( $http_options );
                $cert_content = file_get_contents( $cert_url, FALSE, $cert_stream_context );

                // obtener llave publica
                $pubkeyid = openssl_pkey_get_public( $cert_content );

                // verificar firma
                $resultado = openssl_verify( $data_signed, $notification_signature, $pubkeyid, $auth_algo );

                // liberar llave publica
                openssl_free_key( $pubkeyid );

                // verificacion
                if ( $resultado == 1 ) {
                    $order = new WC_Order( $obj_data->resource->ern );
                    
                    // 'processing'
                    if ( $order->status == 'processing' ) {
                        $successful = true;
                    }
                    
                    if ( in_array( $order->status, array( 'on-hold', 'pending' ) ) ) {
                        $this->save_to_log( $obj_data->resource->ern, 'Procesando WebHook - ' . $obj_data->resource->status);
                        switch ( $obj_data->resource->status ) {
                            case 'REGISTERED':
                                // Registrada no hacer nada
                                $successful = true;
                                break;
                            case 'COMPLETED':
                                // Completar la transaccion
                                $order->add_order_note( __( 'Pago Completado, referencia de pago: ', 'woocommerce-gateway-pagadito' ) . $obj_data->resource->reference, 1 );
                                $this->save_to_log( $order->get_order_number(), 'Referencia de pago = ' . $obj_data->resource->reference );
                                $order->payment_complete( $obj_data->resource->reference );
                                $successful = true;
                                break;
                            case 'VERIFYING':
                                // Verificacion porner la transaccion on hold
                                if ( $order->status == 'on-hold' ) {
                                    // Verifica que la transaccion no haya sido marcada previamente on hold
                                    break;
                                }
                                $order->add_order_note( __( 'Pago en Verificaci&oacute;n, referencia de pago: ', 'woocommerce-gateway-pagadito' ) . $obj_data->resource->reference, 1 );
                                $order->update_status( 'wc-on-hold', 'El pago quedo en verificación.' );
                                $this->save_to_log( $order->get_order_number(), 'Referencia de pago = ' . $obj_data->resource->reference );
                                $successful = true;
                                break;
                            default:
                                // Poderecto tomar la transaccion como fallida
                                // REVOKED, FAILED, CANCELED, EXPIRED
                                $order->update_status( 'wc-failed', 'Pago cancelado por el usuario.' );
                                $this->save_to_log( $order->get_order_number(), 'Referencia de pago = ' . $obj_data->resource->reference );
                                $successful = true;
                                break;
                        }
                    } elseif ( $order->status == 'processing' ){
                        // En este punto el pago ya habia sido registrado y al transaccion
                        // completada, devuelve codigo 200 para que pagadito no vuelva
                        // a enviar el webhook
                        $successful = true;
                    } else {
                        $this->save_to_log( $obj_data->resource->ern, 'Orden en estado diferente a on-hold o pending' );
                    }
                } elseif ( $resultado == 0 ) {
                    $this->save_to_log( 'NA', 'WH verificacion de la firma invalida' );
                } else {
                    $this->save_to_log( 'NA', 'WH error realizando la verificacion de la firma' );
                }
                
                if ( $successful ) {
                    header('HTTP/1.1 200 OK');
                } else {
                    header('HTTP/1.1 427 Nope, something is not right');
                }
                exit;
            }
            
            /**
             * Verificar Pago
             * Redirecciona a la pantalla de Orden completada y en caso de error direcciona a orden cancelada.
             */
            public function check_pagadito_response(){
                global $woocommerce;
                
                //Estados de transacciones en los que se permite realizar la comprobacion de pago
                $allowed_transaction_states = Array(
                    'pending',
                    'processing',
                );
                
                $values = stripslashes_deep( $_GET );
                $token_pagadito = '';
                try{
                    if ( ! isset( $values['token'], $values['order_id'] ) )
                        throw new Exception( __( 'Se intento verificar una transaccion con token u order_id nulo', 'woocommerce-gateway-pagadito' ) );

                    //Cargando la orden
                    $order = new WC_Order ( $values['order_id'] );
                    $this->save_to_log( $order->get_order_number(), '...Regreso de Pagadito' );
                    if ( ! in_array( $order->status, $allowed_transaction_states ) )
                        throw new Exception( __( 'La transaccion no posee un estado valido para ser procesada: ', 'woocommerce-gateway-pagadito' ) . $order->status );
                    
                    // Redireccionar a pagina de gracias en caso que la orden ya haya sido procesada
                    if ( $order->status == 'processing'){
                        wp_redirect( html_entity_decode ( $order->get_checkout_order_received_url() ) );
                        return;
                    }
                    
                    // Obtener el token pagadito, por la URL o por el campo personalizado
                    if ( ! empty( $values['token'] ) ) {
                        $token_pagadito = $values[ 'token' ];
                    } else {
                        $token_pagadito = end( get_post_meta( $order->get_id(), '_token_pagadito' ) );
                    }
                    
                    if ( $token_pagadito === false )
                        throw new Exception( __( 'No se encontro el Token Pagadito', 'woocommerce-gateway-pagadito' ) );
                    
                    if ( $token_pagadito != $values[ 'token' ] )
                        throw new Exception( __( 'Token Pagadito asociado a la orden no corresponde con el parametro get: ', 'woocommerce-gateway-pagadito' ) . $token_pagadito . ' != ' . $values[ 'token' ] );
                    
                    $pg =  new Pagadito($this->uid, $this->wsk );
                    
                    if ( $this->sandbox_mode == 'yes' ){
                        $pg->mode_sandbox_on();
                        $this->save_to_log( $order->get_order_number(), 'Modo SandBox activado' );
                    }
                    
                    if ( ! $pg->connect() )
                        throw new Exception( 'Connect (' . $pg->get_rs_code() . '): ' . $pg->get_rs_message() );

                    if ( ! $pg->get_status( $token_pagadito ) )
                        throw new Exception( 'Get Status (' . $pg->get_rs_code() . '): ' . $pg->get_rs_message() );
                    $respuesta_pg = $pg->get_rs_status();
                    $num_aprob_pg = $pg->get_rs_reference();
                    $this->save_to_log( $order->get_order_number(), 'Comprobando pago, estado = ' . $respuesta_pg );

                    switch ( $respuesta_pg ) {
                        case 'COMPLETED':
                            $order->add_order_note( __( 'Pago Completado, referencia de pago: ', 'woocommerce-gateway-pagadito' ) . $pg->get_rs_reference(), 1 );
                            $this->save_to_log( $order->get_order_number(), 'Referencia de pago = ' . $pg->get_rs_reference() );
                            $order->payment_complete( $pg->get_rs_reference() );
                            wp_redirect( html_entity_decode( $order->get_checkout_order_received_url() ) );
                            return;
                            break;
                        case 'VERIFYING':
                            $order->add_order_note( __( 'Pago en Verificaci&oacute;n, referencia de pago: ' , 'woocommerce-gateway-pagadito' ) . $pg->get_rs_reference(), 1 );
                            $order->update_status( 'wc-on-hold', __( 'El pago quedo en verificaci&oacute;n.', 'woocommerce-gateway-pagadito' ) );
                            $this->save_to_log( $order->get_order_number(), 'Referencia de pago = ' . $pg->get_rs_reference() );
                            wp_redirect( html_entity_decode( $order->get_checkout_order_received_url() ) );
                            return;
                            break;
                        default:
                            $order->add_order_note( __( 'Transacci&oacute;n no completada, estado: ', 'woocommerce-gateway-pagadito' ) . $respuesta_pg );
                            wp_redirect( html_entity_decode( $order->get_cancel_order_url() ) );
                            return;
                            break;
                    }
                    exit;
                } catch ( Exception $e ){
                    $this->save_to_log( $order->get_order_number(), 'Verificando pago - ' . $e->getMessage() );
                    $order->add_order_note( __( 'Error en el procesamiento de la orden: ', 'woocommerce-gateway-pagadito' ) . $e->getMessage() );

                    //Redireccionando a pantalla de error
                    wp_redirect( html_entity_decode( $order->get_cancel_order_url() ) );
                    return;
                }
                    
                return false;
            }
            
            public function get_return_url( $order = '', $page = 'thanks' ) {
                $thanks_page_id = woocommerce_get_page_id( $page );
                if ( $thanks_page_id ) :
                    $return_url = get_permalink( $thanks_page_id );
                else :
                    $return_url = home_url();
                endif;

                if ( $order ) :
                    $return_url = add_query_arg( 'key', $order->order_key, add_query_arg( 'order', $order->get_id(), $return_url ) );
                endif;

                if ( is_ssl() || get_option( 'woocommerce_force_ssl_checkout' ) == 'yes' )
                    $return_url = str_replace( 'http:', 'https:', $return_url );

                return apply_filters( 'woocommerce_get_return_url', $return_url );
            }
        }
    endif;

    /**
     * Agregando Pagadito a WooCommerce
     */
    function add_pagadito_gateway( $methods ) {
        $methods[] = 'WC_Gateway_Pagadito';
        return $methods;
    }

    /**
     * Agregando enlaces adicionales al Plugin
     */
    function plugin_action_links( $links ) {
        $links[] = '<a href="admin.php?page=wc-settings&tab=checkout&section=pagadito">' . esc_html__( 'Configuraci&oacute;n', 'woocommerce-gateway-pagadito' ) . '</a>';
        $links[] = '<a href="https://dev.pagadito.com/index.php?mod=docs" target="_blank">' . esc_html__( 'Documentaci&oacute;n', 'woocommerce-gateway-pagadito' ) . '</a>';
        
        return $links;
    }

    /**
     * Muestra el Numero de Aprobacion PG
     */
    function woo_change_order_received_text( $str, $order ) {
        $str = __( 'NUMERO DE APROBACION PAGADITO: <strong>', 'woocommerce-gateway-pagadito' ) . $order->transaction_id . '</strong>';
        
        return $str;
    }

    add_filter( 'woocommerce_payment_gateways', 'add_pagadito_gateway' );
    add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), 'plugin_action_links' );
    add_filter( 'woocommerce_thankyou_order_received_text', 'woo_change_order_received_text', 10, 2 );
}
